CON

  {Normal QTI Calibration}

  outerLeftThreshold = 2200
  leftThreshold = 1000
  middleThreshold = 700
  rightThreshold = 1100
  outerRightThreshold = 1000

  {Seesaw QTI Calibration}
  
  seeSawOuterLeftThreshold = 2509
  seeSawLeftThreshold = 701
  seeSawMiddleThreshold = 900
  seeSawRightThreshold = 1300
  seeSawOuterRightThreshold = 1500

  {Sensitivity}
  
  Sensitivity = 30

  {Normal CPAL Calibration}

  leftBlackRed = 0
  leftBlackGreen = 0
  leftBlackBlue = 0

  leftGreenRed = 33
  leftGreenGreen = 64
  LeftGreenBlue = 70

  leftSilverRed = 0
  leftSilverGreen = 0
  leftSilverBlue = 0

  leftWhiteRed = 105
  leftWhiteGreen = 81
  leftWhiteBlue = 131

  rightBlackRed = 0
  rightBlackGreen = 0
  rightBlackBlue = 0

  rightGreenRed = 36
  rightGreenGreen = 53
  rightGreenBlue = 73
  
  
  rightSilverRed = 0
  rightSilverGreen = 0
  rightSilverBlue = 0

  rightWhiteRed = 0
  rightWhiteGreen = 0
  rightWhiteBlue = 0

  {Misc}

  CS  = 27                                                       ' H48C connections
  CLK = 25
  DIO = 26

  FastSyncSpeed = 8
  SlowSyncSpeed = 18
  MediumSyncSpeed = 35

  DegreeIncrement = 10000/360

OBJ

  QTI           : "QTIDriver"

  CPALLeft      : "ColorPAL"
  CPALRight     : "ColorPAL"

  Gyro          : "Gyroscope"

  Profiler      : "Profiler"
  Profiler1     : "Profiler"
  Profiler2     : "Profiler"
  Profiler3     : "Profiler"
  Profiler4     : "Profiler"
  Profiler5     : "Profiler"
  
  Ping          : "Ping"
  
VAR

  byte LeftQTIPin, RightQTIPin, MiddleQTIPin, LeftColorPALPin, RightColorPALPin, outerRightQTIPin, outerLeftQTIPin

  long cpallr, cpallg, cpallb
  long cpalrr, cpalrg, cpalrb

  byte cog, fastUpdateMs, fastUpdateMsFull, fastWaitMs
  byte cog2, mediumUpdateMs, mediumUpdateMsFull, mediumWaitMs
  byte cog1, slowUpdateMs, slowUpdateMsFull, slowWaitMs 
  long stack[75]

  long LeftQTI, RightQTI, MiddleQTI, OuterLeftQTI, OuterRightQTI, CPALLeftPin, CPALRightPin, X, Y, Z, Degrees

  long LQC, RQC, MQC, OLQC, ORQC

  long cmm, cmr, mmm, mmr

PUB Start(CPALPinLeft, QTIOuterLeftPin, QTILeftPin, QTIMiddlePin, QTIRightPin, QTIOuterRightPin, CPALPinRight)

  {Set Pin Variables} 

  CPALLeftPin := CPALPinLeft
  CPALRightPin := CPALPinRight
  LeftQTIPin := QTILeftPin
  MiddleQTIPin := QTIMiddlePin
  RightQTIPin := QTIRightPin
  LeftColorPALPin := CPALLeftPin
  RightColorPALPin := CPALRightPin
  OuterLeftQTIPin := QTIOuterLeftPin
  OuterRightQTIPin := QTIOuterRightPin

  {Initalize Everything Else}

  SetNormalCalib
  InitButtons
  LaunchCogs

PUB LaunchCogs

  cog := cognew(FastUpdateLoop, @stack[0])
  cog1 := cognew(SlowUpdateLoop, @stack[25])
  cog2 := cognew(MediumUpdateLoop, @stack[50])

PUB InitButtons

  DIRA[16..18]~    

PUB SetNormalCalib

  OLQC := outerLeftThreshold
  LQC := leftThreshold
  MQC := middleThreshold
  RQC := rightThreshold
  ORQC := outerRightThreshold

PUB SetSeesawCalib

  OLQC := seeSawOuterLeftThreshold
  LQC := seeSawLeftThreshold
  MQC := seeSawMiddleThreshold
  RQC := seeSawRightThreshold
  OLQC := seeSawOuterRightThreshold  

PUB GyroStart

  Gyro.Start

PUB Update

  Gyro.Update

PUB GetGyroX

  return Gyro.GetX

PUB GetGyroY

  return Gyro.GetY

PUB GetGyroZ

  return Gyro.GetZ

PUB Stop

  cogstop(cog)

PUB FastUpdateLoop

  QTI.addQTI(LeftQTIPin)
  QTI.addQTI(RightQTIPin)
  QTI.addQTI(OuterLeftQTIPin)
  QTI.addQTI(OuterRightQTIPin)
  QTI.addQTI(MiddleQTIPin)

  
  '---[Comment Out This Code For More Speed]---
  '--------------------------------------------'

  repeat
    Profiler1.StartTimer
    Profiler.StartTimer

    'Update Sensor Variables
    LeftQTI := QTI.GetRaw(LeftQTIPin)
    MiddleQTI := QTI.getRaw(MiddleQTIPin)
    RightQTI := QTI.getRaw(RightQTIPin)
    OuterLeftQTI := QTI.getRaw(OuterLeftQTIPin)
    OuterRightQTI := QTI.getRaw(OuterRightQTIPin)

    '---[Comment Out This Code For More Speed]---'
    {Gyro.Update
    X := Gyro.GetX
    Y := Gyro.GetY
    Z := Gyro.GetZ
    Degrees := Z/DegreeIncrement }
    '--------------------------------------------'

    {cmm := Ping.Centimeters(6)
    cmr := Ping.Centimeters(7)
    cmm <#= 100
    cmr <#= 100 }

    Profiler.StopTimer
    fastupdateMs := Profiler.getTimeMs
    if fastupdateMs < FastSyncSpeed
      fastwaitMs := FastSyncSpeed - fastupdateMs
      pause(fastwaitMs)
    else
      fastwaitMs := -1
    Profiler1.StopTimer
    fastupdateMsFull := Profiler1.getTimeMs

PUB SlowUpdateLoop

  CPALLeft.Init(CPALLeftPin)
  CPALRight.Init(CPALRightPin) 

  repeat
    Profiler3.StartTimer
    Profiler2.StartTimer

    cpallr := CPALLeft.GetRed
    cpallg := CPALLeft.GetGreen
    cpallb := CPALLeft.GetBlue

    cpalrr := CPALRight.GetRed
    cpalrg := CPALRight.GetGreen
    cpalrb := CPALRight.GetBlue

    Profiler2.StopTimer
    slowupdateMs := Profiler2.getTimeMs
    if slowupdateMs < SlowSyncSpeed
      slowwaitMs := SlowSyncSpeed - SlowupdateMs
      pause(SlowwaitMs)
    else
      SlowwaitMs := -1
    Profiler3.StopTimer
    SlowupdateMsFull := Profiler3.getTimeMs

PUB MediumUpdateLoop

  Gyro.Start 

  repeat
    Profiler5.StartTimer
    Profiler4.StartTimer

    Gyro.Update
    X := Gyro.GetX
    Y := Gyro.GetY
    Z := Gyro.GetZ
    Degrees := Z/DegreeIncrement

    mmm := Ping.Millimeters(6)
    mmr := Ping.Millimeters(7)
    cmm := Ping.Centimeters(6)
    cmr := Ping.Centimeters(7)
    cmm <#= 100
    cmr <#= 100

    Profiler4.StopTimer
    mediumupdateMs := Profiler4.getTimeMs
    if mediumupdateMs < mediumSyncSpeed
      mediumwaitMs := mediumSyncSpeed - mediumupdateMs
      pause(mediumwaitMs)
    else
      mediumwaitMs := -1
    Profiler5.StopTimer
    mediumupdateMsFull := Profiler5.getTimeMs
  

PUB GetFastWait

  return fastWaitMs

PUB GetSlowWait

  return slowWaitMs

PUB GetFastLastMs

  return fastUpdateMs

PUB GetSlowLastMs

  return slowUpdateMs

PUB GetFastLastMsFull

  return fastupdateMsFull

PUB GetSlowLastMsFull

  return slowupdateMsFull

PUB GetMediumWait

  return mediumWaitMs

PUB getMediumLastMs

  return mediumUpdateMs

PUB getMediumLastMsFull

  return mediumUpdateMsFull

PUB IsFastLoopSlowerThanSync

  if FastupdateMs > FastSyncSpeed
    return true
  else
    return false

PUB IsSlowLoopSlowerThanSync

  if SlowupdateMs > SlowSyncSpeed
    return true
  else
    return false

PUB GetX

  return X

PUB GetY

  return Y

PUB GetZ

  return Z

PUB GetDegrees

  return Degrees

{PUB Update

  Gyro.Update}

PUB GetCmMiddle

  return cmm

PUB GetCmRight

  return cmr

PUB GetMMMiddle

  return mmm

PUB GetMMRight

  return mmr

PUB getLeftCPALColor

  'if cpallr > leftBlackRed - Sensitivity and cpallr < leftBlackRed + Sensitivity and cpallg > leftBlackGreen - Sensitivity and cpallg < leftBlackGreen + Sensitivity and cpallb > leftBlackBlue - Sensitivity and cpallb < leftBlackBlue + Sensitivity 
    'return string("Black")
  if cpallr > leftGreenRed - Sensitivity and cpallr < leftGreenRed + Sensitivity and cpallg > leftGreenGreen - Sensitivity and cpallg < leftGreenGreen + Sensitivity and cpallb > leftGreenBlue - Sensitivity and cpallb < leftGreenBlue + Sensitivity
    return string("Green")
  'elseif cpallr > leftSilverRed - Sensitivity and cpallr < leftSilverRed + Sensitivity and cpallg > leftSilverGreen - Sensitivity and cpallg < leftSilverGreen + Sensitivity and cpallb > leftSilverBlue - Sensitivity and cpallb < leftSilverBlue + Sensitivity
    'return string("Silver")
  else'if cpallr > leftWhiteRed - Sensitivity and cpallr < leftWhiteRed + Sensitivity and cpallg > leftWhiteGreen - Sensitivity and cpallg < leftWhiteGreen + Sensitivity and cpallb > leftWhiteBlue - Sensitivity and cpallb < leftWhiteBlue + Sensitivity
    return string("White")

PUB getRightCPALColor

  'if cpallr > rightBlackRed - Sensitivity and cpallr < rightBlackRed + Sensitivity and cpallg > rightBlackGreen - Sensitivity and cpallg < rightBlackGreen + Sensitivity and cpallb > rightBlackBlue - Sensitivity and cpallb < rightBlackBlue + Sensitivity 
    'return string("Black")
  if cpalrr > rightGreenRed - Sensitivity and cpalrr < rightGreenRed + Sensitivity and cpalrg > rightGreenGreen - Sensitivity and cpalrg < rightGreenGreen + Sensitivity and cpalrb > rightGreenBlue - Sensitivity and cpalrb < rightGreenBlue + Sensitivity
    return string("Green")
  'elseif cpallr > rightSilverRed - Sensitivity and cpallr < rightSilverRed + Sensitivity and cpallg > rightSilverGreen - Sensitivity and cpallg < rightSilverGreen + Sensitivity and cpallb > rightSilverBlue - Sensitivity and cpallb < rightSilverBlue + Sensitivity
    'return string("Silver")
  else'if cpallr > rightWhiteRed - Sensitivity and cpallr < rightWhiteRed + Sensitivity and cpallg > rightWhiteGreen - Sensitivity and cpallg < rightWhiteGreen + Sensitivity and cpallb > rightWhiteBlue - Sensitivity and cpallb < rightWhiteBlue + Sensitivity
    return string("White")    

PUB getLeftQTI

  return LeftQTI

PUB getMiddleQTI

  return MiddleQTI

PUB getRightQTI

  return RightQTI

PUB getOuterLeftQTI

  return OuterLeftQTI
  
PUB getOuterRightQTI

  return OuterRightQTI

PUB getLeftQTIColor

  if LeftQTI > LQC
    return 1
  else
    return 0

PUB getRightQTIColor

  if RightQTI > RQC
    return 1
  else
    return 0

PUB getMiddleQTIColor

  if MiddleQTI > MQC
    return 1
  else
    return 0

PUB getOuterLeftQTIColor

  if OuterLeftQTI > OLQC
    return 1
  else
    return 0

PUB getOuterRightQTIColor

  if OuterRightQTI > ORQC
    return 1
  else
    return 0

PUB getLeftRed

  return cpallr

PUB getLeftGreen

  return cpallg

PUB getLeftBlue

  return cpallb

PUB getRightRed

  return cpalrr 

PUB getRightGreen

  return cpalrg

PUB getRightBlue

  return cpalrb

PUB GetArmButton

  return INA[16]

PUB GetButtonLeft

  return INA[17]

PUB GetButtonRight

  return INA[18]

PUB getOuterLeftCalibration

  return OLQC

PUB getLeftCalibration

  return LQC

PUB getMiddleCalibration

  return MQC

PUB getRightCalibration

  return RQC

PUB getOuterRightCalibration

  return ORQC

PRI Pause(ms)

  waitcnt(clkfreq/(1000/ms)+cnt)
                                